/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.process.ImageFilterType
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.io.File

class PageFileStoragePlugin(
    val activity: Activity,
    coroutineResultHandler: CoroutineResultHandler,
    val scanbotSDK: ScanbotSDK
) : ISdkSubplugin,
    ICoroutineActivityResultHandler by coroutineResultHandler {


    private val gson = GsonBuilder()
        .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
        .create()
    private val pageFileStorage = scanbotSDK.pageFileStorage()
    private val pageProcessor = scanbotSDK.pageProcessor()

    override suspend fun onMethodCall(call: MethodCall, result: Result): Boolean {

        when (call.method) {
            "getFilteredDocumentPreviewUri" -> getFilteredDocumentPreviewUri(activity, call, result)
            "deletePage" -> deletePage(activity, call, result)
            "cleanupStorage" -> cleanupStorage(activity, call, result)
            else -> return false
        }
        return true
    }


    private suspend fun getFilteredDocumentPreviewUri(
        activity: Activity,
        call: MethodCall,
        methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<PageJson>("page") ?: PageJson()
                val page = pageJson.toPage(gson)

                val imageFilterType = call.argument<String>("filter") ?: ""
                val filter = ImageFilterType.valueOf(imageFilterType)

                val previewFileUri = pageFileStorage.getFilteredPreviewImageURI(page.pageId, filter)
                if (!File(previewFileUri.path).exists()) {
                    pageProcessor.generateFilteredPreview(page, filter)
                }

                withContext(Dispatchers.Main) { methodResult.success(previewFileUri.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                        e::class.java.simpleName,
                        e.message,
                        null
                    )
                }
            }
        }
    }


    private suspend fun deletePage(
        activity: Activity,
        call: MethodCall,
        methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<PageJson>("page") ?: PageJson()
                val page = pageJson.toPage(gson)

                val pageId = page.pageId ?: ""
                val deleted = pageFileStorage.remove(pageId)
                if (deleted) {
                    withContext(Dispatchers.Main) { methodResult.success("") }
                } else {
                    withContext(Dispatchers.Main) {
                        methodResult.error(
                            "PageException",
                            "Could not delete page",
                            null
                        )
                    }
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                        e::class.java.simpleName,
                        e.message,
                        null
                    )
                }
            }
        }
    }


    private suspend fun cleanupStorage(
        activity: Activity,
        call: MethodCall,
        methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                // cleanup the pages storage
                pageFileStorage.removeAll()

                // cleanup the plugin storage folder (generated TIFF, PDF, etc)
                StorageUtils.cleanupPluginStorageDirectory(activity)

                withContext(Dispatchers.Main) { methodResult.success("") }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                        e::class.java.simpleName,
                        e.message,
                        null
                    )
                }
            }
        }
    }
}

