/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import android.net.Uri
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.JsonArray
import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.persistence.Page
import io.scanbot.sdk.persistence.PageFileStorage
import io.scanbot.sdk.process.ImageFilterType
import io.scanbot.sdk.process.PDFPageSize
import io.scanbot.tiffwriter.model.TIFFImageWriterCompressionOptions.*
import io.scanbot.tiffwriter.model.TIFFImageWriterParameters
import io.scanbot.tiffwriter.model.TIFFImageWriterUserDefinedField
import kotlinx.coroutines.*
import net.doo.snap.entity.Language
import net.doo.snap.lib.detector.DetectionResult
import org.json.JSONObject
import java.io.File
import java.util.ArrayList

class PageProcessorPlugin(
        val activity: Activity,
        coroutineResultHandler: CoroutineResultHandler,
        val scanbotSDK: ScanbotSDK
) : ISdkSubplugin,
        ICoroutineActivityResultHandler by coroutineResultHandler {

    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private val pageProcessor = scanbotSDK.pageProcessor()
    private val ocrRecognizer = scanbotSDK.ocrRecognizer()
    private val pageFileStorage = scanbotSDK.pageFileStorage()
    private val pageFactory = scanbotSDK.pageFactory()
    override suspend fun onMethodCall(call: MethodCall, result: Result): Boolean {
        when (call.method) {
            "applyImageFilter" -> applyImageFilter(activity, call, result)
            "rotatePageClockwise" -> rotatePageClockwise(activity, call, result)
            "performOcr" -> performOcr(activity, call, result)
            "getOcrConfig" -> getOcrConfigs(activity, call, result)
            "detectDocument" -> detectDocument(activity, call, result)
            "createPage" -> createPage(activity, call, result)
            "createTiff" -> createTiff(activity, call, result)
            "createPdf" -> createPdf(activity, call, result)
            else -> return false
        }
        return true
    }

    private suspend fun applyImageFilter(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<PageJson>("page") ?: PageJson()
                val imageFilterType = call.argument<String>("filter") ?: ""
                val pageFromJson = pageJson.toPage(gson)

                val page = pageProcessor.applyFilter(
                        pageFromJson,
                        ImageFilterType.valueOf(imageFilterType)
                )
                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) { methodResult.success(json.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun rotatePageClockwise(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson =
                        call.argument<PageJson>("page") ?: PageJson()
                val times =
                        call.argument<Int>("rotationsCount") ?: 1

                val pageFromJson = pageJson.toPage(gson)

                val page = pageProcessor.rotate(
                        pageFromJson,
                        times
                )
                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)

                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun detectDocument(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson =
                        call.argument<PageJson>("page") ?: PageJson()
                var page = pageJson.toPage(gson)

                // run auto document detection on it:
                page = pageProcessor.detectDocument(page)

                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createPage(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val originalImageFileUri = call.argument<String>("originalImageFileUri") ?: ""
                val shouldDetectDocument = call.argument<Boolean>("shouldDetectDocument") ?: false

                val originalBitmap = prepareImageForImport(Uri.parse(originalImageFileUri), activity, fixOrientation = true)
                val pageId = pageFileStorage.add(originalBitmap)
                var page = Page(pageId, emptyList(), DetectionResult.OK, ImageFilterType.NONE)

                if (shouldDetectDocument) {
                    // run auto document detection on it:
                    page = pageProcessor.detectDocument(page)
                }

                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun getOcrConfigs(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val configJson = JsonObject()
                val languagesJson = JsonArray()
                ocrRecognizer.getInstalledLanguages().map { it.iso1Code }.forEach {
                    languagesJson.add(it)
                }
                configJson.addProperty("languageDataPath", ocrRecognizer.getLanguageDataPath().path)
                configJson.add("installedLanguages", languagesJson)
                withContext(Dispatchers.Main) {
                    methodResult.success(configJson.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun performOcr(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<OcrOptionsJson>("options")

                val languages = (options?.get("languages") as LanguagesJson).mapNotNull {
                    Language.languageByIso(it)
                }.toSet()

                val shouldGeneratePdf = options?.get("shouldGeneratePdf") as? Boolean
                val pages =
                        pagesJson.map { gson.fromJson(JSONObject(it).toString(), Page::class.java) }
                                .toList()

                val ocrResult = if (shouldGeneratePdf == true) {
                    ocrRecognizer.recognizeTextWithPdfFromPages(pages, PDFPageSize.AUTO, languages)
                } else {
                    ocrRecognizer.recognizeTextFromPages(pages, languages)
                }
                val tmpFile = ocrResult?.sandwichedPdfDocumentFile
                val newFile = tmpFile?.let {
                    val newFile = movePdfFileToStorage(it)
                    newFile
                }

                val ocrResultJson = ocrResultToJson(ocrResult, newFile)
                withContext(Dispatchers.Main) {
                    methodResult.success(ocrResultJson.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createTiff(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<TiffOptionsJson>("options")
                val binarized = options?.get("binarized") as Boolean
                val compressionString = (options?.get("compression") as? String)
                        .takeUnless { it.isNullOrBlank() }
                        ?: "NONE"
                val imageFiles =
                        pagesJson.map { gson.fromJson(JSONObject(it).toString(), Page::class.java) }
                                .map {
                                    async {
                                        pageFileStorage.getImageURI(
                                                it.pageId,
                                                PageFileStorage.PageFileType.DOCUMENT
                                        )
                                    }
                                }.awaitAll().map {
                                    File(it.path)
                                }


                val dpi = (options?.get("dpi") as? Int) ?: 200

                val compression = valueOf("COMPRESSION_$compressionString")


                // no custom fields supported yet.
                val customFields = ArrayList<TIFFImageWriterUserDefinedField>()

                val params = TIFFImageWriterParameters(binarized, dpi, compression, customFields)

                val tiffTargetFile = StorageUtils.generateRandomPluginStorageFile("tiff", activity)
                val tiffWriter = scanbotSDK.tiffWriter()
                tiffWriter.writeTIFFFromFiles(imageFiles, tiffTargetFile, params)

                withContext(Dispatchers.Main) {
                    methodResult.success(Uri.fromFile(tiffTargetFile).toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createPdf(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<PdfOptionsJson>("options")
                val value = (options?.get("renderPageSize") as? String) ?: PDFPageSize.AUTO.name
                val pdfPageSize = PDFPageSize.valueOf(value)
                val pages = pagesJson.map { gson.fromJson(JSONObject(it).toString(), Page::class.java) }
                val pdfRenderer = scanbotSDK.pdfRenderer()
                val tmpfile = pdfRenderer.renderDocumentFromPages(pages, pdfPageSize)
                val file = movePdfFileToStorage(tmpfile)
                withContext(Dispatchers.Main) {
                    methodResult.success(Uri.fromFile(file).toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun movePdfFileToStorage(tempPdfFile: File?): File {
        try {
            val pdfOutputUri = Uri.fromFile(StorageUtils.generateRandomPluginStorageFile("pdf", activity))
            val newFile = File(pdfOutputUri.path)
            tempPdfFile?.let { StorageUtils.copyFile(it, newFile) }
            return newFile
        } catch (e: Throwable) {
            throw e
        } finally {
            tempPdfFile?.let { StorageUtils.forceDelete(it) }
        }
    }
}

