/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import android.net.Uri
import android.util.Log
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.ScanbotSDKInitializer
import io.scanbot.sdk.barcode.ScanbotBarcodeDetector
import io.scanbot.sdk.persistence.CameraImageFormat
import io.scanbot.sdk.persistence.PageStorageSettings
import kotlinx.coroutines.*
import java.io.File

class ScanbotSdkPlugin(val activity: Activity, val registrar: Registrar) : MethodCallHandler,
        CoroutineResultHandler(), ISdkSubpluginManager by SdkSubpluginManager() {

    companion object {

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val sdkChannel = MethodChannel(registrar.messenger(), "scanbot_sdk")
            val scanbotSdkPlugin = ScanbotSdkPlugin(registrar.activity(), registrar)
            sdkChannel.setMethodCallHandler(scanbotSdkPlugin)
            registrar.addActivityResultListener(scanbotSdkPlugin)
        }
    }

    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private var job: Job = Job()
    private lateinit var scanbotSDK: ScanbotSDK


    override fun onMethodCall(call: MethodCall, result: Result) {
        GlobalScope.launch(Dispatchers.IO) {
            when (call.method) {
                "initScanbotSdk" -> {
                    initScanbotSdk(activity, call, result)
                }
                "getLicenseStatus" -> {
                    isLicenseValid(activity, call, result)
                }
                else -> {
                    if (!callMethod(call, result)) {
                        withContext(Dispatchers.Main) { result.notImplemented() }
                    }
                }
            }
        }
    }

    private suspend fun initScanbotSdk(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Main) {
            try {
                val licenseKey = call.argument<String>("licenseKey") ?: ""
                val scanFormat = call.argument<String>("imageFormat") ?: ""
                val storageBaseDirectory = call.argument<String>("storageBaseDirectory")
                val imageQuality = call.argument<Int>("imageQuality") ?: 80
                val previewTargetMax = call.argument<Int>("previewTargetMax") ?: 900
                val loggingEnabled = call.argument<Boolean>("loggingEnabled") ?: false
                val imageFormat = if (scanFormat.contains("PNG")) {
                    CameraImageFormat.PNG
                } else {
                    CameraImageFormat.JPG
                }

                val scanbotSDKInitializer = ScanbotSDKInitializer()

                if (!storageBaseDirectory.isNullOrEmpty()) {
                    val uri = Uri.parse(storageBaseDirectory)
                    val customStorageBaseDirectory = File(uri.path)
                    customStorageBaseDirectory.mkdirs()
                    if (customStorageBaseDirectory.isDirectory.not()) {
                        methodResult.error(
                                "IOException",
                                "Specified storageBaseDirectory is not valid: $uri",
                                null
                        )
                    }
                    scanbotSDKInitializer.sdkFilesDirectory(
                            activity.application,
                            customStorageBaseDirectory
                    )
                    StorageUtils.setCustomStorageBaseDirectory(customStorageBaseDirectory)
                    Log.d(
                            "SDK",
                            "Using custom storage base directory: " + customStorageBaseDirectory.absolutePath
                    )
                }



                scanbotSDKInitializer
                        .useBarcodeDetector(ScanbotBarcodeDetector.BarcodeDetectorType.ZXing)
                        .withLogging(loggingEnabled)
                        .usePageStorageSettings(
                                PageStorageSettings.Builder()
                                        .imageFormat(imageFormat)
                                        .imageQuality(imageQuality)
                                        .previewTargetMax(previewTargetMax)
                                        .build()
                        )
                        .prepareOCRLanguagesBlobs(true)
                        .prepareMRZBlobs(true)
                        .preparePayFormBlobs(true)
                        .license(activity.application, licenseKey)
                        .initialize(activity.application)
                scanbotSDK = ScanbotSDK(activity.application)

                clearPlugins()

                registerSubplugin(
                        ScanbotSdkUiPlugin(
                                registrar.activity(),
                                this@ScanbotSdkPlugin,
                                scanbotSDK
                        )
                )
                registerSubplugin(
                        PageFileStoragePlugin(
                                registrar.activity(),
                                this@ScanbotSdkPlugin,
                                scanbotSDK
                        )
                )
                registerSubplugin(
                        PageProcessorPlugin(
                                registrar.activity(),
                                this@ScanbotSdkPlugin,
                                scanbotSDK
                        )
                )
                if ("" != licenseKey.trim() && "null" != licenseKey.toLowerCase()) {
                    methodResult.success("Scanbot SDK initialized.")
                } else {
                    methodResult.success(
                            "Scanbot SDK initialized. Trial mode activated. You can now test all features for 60 seconds."
                    )
                }
            } catch (e: Throwable) {
                Log.e("SDK", e.toString())
                methodResult.error(e::class.java.simpleName, e.message, null)
            }
        }
    }

    private suspend fun isLicenseValid(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Main) {
            try {
                if (::scanbotSDK.isInitialized) {
                    val json = JsonObject()
                    json.addProperty("isLicenseValid", scanbotSDK.isLicenseValid)
                    methodResult.success(json.toString())
                } else {
                    methodResult.error(
                            "SDKNotInitialisedException",
                            "need to initialise sdk with licence",
                            null
                    )
                }
            } catch (e: Throwable) {
                methodResult.error(e::class.java.simpleName, e.message, null)
            }
        }
    }


    private suspend fun dispatchCanceled(methodResult: Result) {
        withContext(Dispatchers.Main) {
            methodResult.error(
                    Activity.RESULT_CANCELED.toString(),
                    "Canceled",
                    null
            )
        }
    }


}

