/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import android.content.Intent
import android.os.Build
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.JsonArray
import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.hicscanner.model.HealthInsuranceCardRecognitionResult
import io.scanbot.mrzscanner.model.MRZRecognitionResult
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.barcode.entity.BarcodeItem
import io.scanbot.sdk.barcode.entity.BarcodeScanningResult
import io.scanbot.sdk.persistence.Page
import io.scanbot.sdk.ui.view.barcode.BarcodeScannerActivity
import io.scanbot.sdk.ui.view.camera.DocumentScannerActivity
import io.scanbot.sdk.ui.view.edit.CroppingActivity
import io.scanbot.sdk.ui.view.hic.HealthInsuranceCardScannerActivity
import io.scanbot.sdk.ui.view.mrz.MRZScannerActivity
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import kotlinx.coroutines.withContext

class ScanbotSdkUiPlugin(
        val activity: Activity,
        coroutineResultHandler: CoroutineResultHandler,
        val scanbotSDK: ScanbotSDK
) : ISdkSubplugin,
        ICoroutineActivityResultHandler by coroutineResultHandler {


    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private var job: Job = Job()


    override suspend fun onMethodCall(call: MethodCall, result: Result): Boolean {
        when (call.method) {
            "startBarcodeScanner" -> {
                proceedBarcodeScanner(activity, call, result)
            }
            "startMrzScanner" -> {
                proceedMRZScanner(activity, call, result)
            }
            "startDocumentScanner" -> {
                proceedDefaultScanner(activity, call, result)
            }
            "startCroppingScreen" -> {
                proceedCroppingScreen(activity, call, result)
            }
            "startEhicScanner" -> {
                proceedEhicScreen(activity, call, result)
            }
            else -> {
                return false
            }
        }
        return true
    }

    private suspend fun proceedBarcodeScanner(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        val barcodeCameraConfiguration = try {
            readBarcodeScannerConfiguration(call, activity)
        } catch (e: Throwable) {
            methodResult.error(e::class.java.simpleName, e.message, null)
            return
        }
        val intent =
                BarcodeScannerActivity.newIntent(activity, barcodeCameraConfiguration)
        val activityResult = withContext(Dispatchers.Main) {
            launchIntent(activity, intent).await()
        }

        activityResult?.let {
            if (it.resultCode == Activity.RESULT_OK) {
                val barcodeData =
                        it.data?.getParcelableExtra<BarcodeScanningResult>(BarcodeScannerActivity.SCANNED_BARCODE_EXTRA)

                barcodeData?.let {
                    val resultMap = mutableMapOf<String, String>()
                    it.barcodeItems.firstOrNull()?.let {
                        resultMap.put("text", it.text)
                        resultMap.put("barcodeFormat", it.barcodeFormat.name)
                    }
                    val result = gson.toJson(resultMap)
                    withContext(Dispatchers.Main) { methodResult.success(result) }
                }
            } else {
                dispatchCanceled(methodResult)
            }
        } ?: withContext(Dispatchers.Main) { methodResult.notImplemented() }
    }

    private suspend fun proceedEhicScreen(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {

            val configuration = try {
                readEhicScannerConfiguration(call, activity)
            } catch (e: Throwable) {
                methodResult.error(e::class.java.simpleName, e.message, null)
                return@withContext
            }
            val intent =
                    HealthInsuranceCardScannerActivity.newIntent(activity, configuration)
            val activityResult = withContext(Dispatchers.Main) {
                launchIntent(activity, intent).await()
            }

            activityResult?.let {
                if (it.resultCode == Activity.RESULT_OK) {

                    val result =
                            it.data?.getParcelableExtra<HealthInsuranceCardRecognitionResult>(
                                    HealthInsuranceCardScannerActivity.EXTRACTED_FIELDS_EXTRA
                            )

                    result?.let {
                        val resultJson = gson.toJsonTree(result) as JsonObject
                        withContext(Dispatchers.Main) { methodResult.success(resultJson.toString()) }
                    } ?: returnEmptyResult(methodResult)
                } else {
                    dispatchCanceled(methodResult)
                }
            } ?: withContext(Dispatchers.Main) { methodResult.notImplemented() }

        }
    }

    private suspend fun proceedCroppingScreen(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            val croppingConfiguration = try {
                readCroppingConfiguration(call, activity, gson)
            } catch (e: Throwable) {
                methodResult.error(e::class.java.simpleName, e.message, null)
                return@withContext
            }
            val intent =
                    CroppingActivity.newIntent(activity, croppingConfiguration)
            val activityResult = withContext(Dispatchers.Main) {
                launchIntent(activity, intent).await()
            }

            activityResult?.let {
                if (it.resultCode == Activity.RESULT_OK) {
                    val resultJson = JsonObject()

                    val page =
                            it.data?.getParcelableExtra<Page>(CroppingActivity.EDITED_PAGE_EXTRA)

                    page?.let {
                        val pageJson = gson.toJsonTree(page) as JsonObject
                        insertPageUris(scanbotSDK, this, page.pageId, pageJson)
                        resultJson.add("page", pageJson)
                        withContext(Dispatchers.Main) { methodResult.success(resultJson.toString()) }
                    } ?: returnEmptyResult(methodResult)
                } else {
                    dispatchCanceled(methodResult)
                }
            } ?: withContext(Dispatchers.Main) { methodResult.notImplemented() }

        }
    }

    private suspend fun returnEmptyResult(methodResult: Result) {
        withContext(Dispatchers.Main) {
            methodResult.error(
                    "ERROR",
                    "Empty camera result",
                    null
            )
        }
    }

    private suspend fun proceedMRZScanner(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        val mrzScannerConfiguration = try {
            readMRZScannerConfiguration(call, activity)
        } catch (e: Throwable) {
            methodResult.error(e::class.java.simpleName, e.message, null)
            return
        }
        val intent =
                MRZScannerActivity.newIntent(activity, mrzScannerConfiguration)
        val activityResult = withContext(Dispatchers.Main) {
            launchIntent(activity, intent).await()
        }
        activityResult?.let {
            if (it.resultCode == Activity.RESULT_OK) {
                val barcodeData =
                        it.data?.getParcelableExtra<MRZRecognitionResult>(MRZScannerActivity.EXTRACTED_FIELDS_EXTRA)
                barcodeData?.let {
                    val result = gson.toJson(it)
                    withContext(Dispatchers.Main) { methodResult.success(result) }
                }
            } else {
                dispatchCanceled(methodResult)
            }
        } ?: withContext(Dispatchers.Main) { methodResult.notImplemented() }
    }


    private suspend fun proceedDefaultScanner(
            activity: Activity,
            call: MethodCall,
            methodResult: Result
    ) {
        val cameraConfiguration = try {
            readDocumentScannerConfiguration(call, activity)
        } catch (e: Throwable) {
            methodResult.error(e::class.java.simpleName, e.message, null)
            return
        }
        val intent = DocumentScannerActivity.newIntent(
                activity,
                cameraConfiguration
        )
        val activityResult = withContext(Dispatchers.Main) {
            launchIntent(activity, intent).await()
        }
        withContext(Dispatchers.Default) {
            activityResult?.let {
                if (it.resultCode == Activity.RESULT_OK) {

                    val resultJson = JsonObject()
                    val array = JsonArray()

                    it.data?.getParcelableArrayExtra(DocumentScannerActivity.SNAPPED_PAGE_EXTRA)
                            ?.map { element ->
                                element as Page
                            }?.map { page ->
                                val pageJson = page.toJson(gson)
                                insertPageUris(scanbotSDK, this, page.pageId, pageJson)
                                pageJson
                            }?.forEach { json ->
                                array.add(json)
                            }?.let {
                                resultJson.add("pages", array)
                                withContext(Dispatchers.Main) { methodResult.success(resultJson.toString()) }
                            } ?: returnEmptyResult(methodResult)

                } else {
                    dispatchCanceled(methodResult)
                }
            } ?: withContext(Dispatchers.Main) { methodResult.notImplemented() }
        }
    }


    private suspend fun dispatchCanceled(methodResult: Result) {
        withContext(Dispatchers.Main) {
            methodResult.error(
                    Activity.RESULT_CANCELED.toString(),
                    "Canceled",
                    null
            )
        }
    }


}

