/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.content.Context
import android.graphics.Bitmap
import android.graphics.Matrix
import androidx.exifinterface.media.ExifInterface
import android.net.Uri
import com.google.gson.JsonObject
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.persistence.PageFileStorage
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.async
import net.doo.snap.util.FileChooserUtils
import net.doo.snap.util.bitmap.BitmapUtils
import java.io.IOException


/**
 * This method converts dp unit to equivalent pixels, depending on device density.
 *
 * @param dp      A value in dp (density independent pixels) unit. Which we need to convert into pixels
 * @param context Context to get resources and device specific display metrics
 * @return A float value to represent px equivalent to dp depending on device density
 */
fun convertDpToPx(context: Context, dp: Float): Float {
    return dp * context.resources.displayMetrics.density
}

suspend fun insertPageUris(
    scanbotSDK: ScanbotSDK,
    coroutineScope: CoroutineScope,
    pageId: String,
    pageJson: JsonObject
) {

    val originalImageFileUri = coroutineScope.async {
        scanbotSDK.pageFileStorage()
            .getImageURI(pageId, PageFileStorage.PageFileType.ORIGINAL)
    }
    val documentImageFileUri = coroutineScope.async {
        scanbotSDK.pageFileStorage()
            .getImageURI(pageId, PageFileStorage.PageFileType.DOCUMENT)
    }
    val originalPreviewImageFileUri = coroutineScope.async {
        scanbotSDK.pageFileStorage()
            .getPreviewImageURI(pageId, PageFileStorage.PageFileType.ORIGINAL)
    }
    val documentPreviewImageFileUri = coroutineScope.async {
        scanbotSDK.pageFileStorage()
            .getPreviewImageURI(
                pageId,
                PageFileStorage.PageFileType.DOCUMENT
            )
    }
    fillPageJsonWithUris(
        pageJson,
        originalImageFileUri.await(),
        documentImageFileUri.await(),
        originalPreviewImageFileUri.await(),
        documentPreviewImageFileUri.await()
    )
}

private suspend fun fillPageJsonWithUris(
    pageJson: JsonObject,
    originalImageFileUri: Uri,
    documentImageFileUri: Uri,
    originalPreviewImageFileUri: Uri,
    documentPreviewImageFileUri: Uri
) {
    pageJson.addProperty(
        "originalImageFileUri",
        (originalImageFileUri.toString())
    )
    pageJson.addProperty(
        "documentImageFileUri",
        (documentImageFileUri.toString())
    )
    pageJson.addProperty(
        "originalPreviewImageFileUri",
       (originalPreviewImageFileUri.toString())
    )
    pageJson.addProperty(
        "documentPreviewImageFileUri",
        (documentPreviewImageFileUri.toString())
    )
}


fun prepareImageForImport(imageUri: Uri, context: Context, fixOrientation: Boolean = true): Bitmap {

    val imagePath = FileChooserUtils.getPath(context, imageUri)
    var bitmap: Bitmap? = BitmapUtils.decodeQuietly(imagePath, null)
            ?: throw IOException("Could not load bitmap by given image file URI: $imageUri")

    if (fixOrientation) {
        val exif = ExifInterface(imagePath)
        val rotation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL)
        val rotationInDegrees = exifToDegrees(rotation)
        val matrix = Matrix()
        if (rotation != 0) {
            matrix.preRotate(rotationInDegrees.toFloat())
        }
        bitmap = Bitmap.createBitmap(bitmap!!, 0, 0, bitmap.width, bitmap.height, matrix, false)
    }

    return bitmap!!
}


private fun exifToDegrees(exifOrientation: Int): Int {
    return when (exifOrientation) {
        ExifInterface.ORIENTATION_ROTATE_90 -> 90
        ExifInterface.ORIENTATION_ROTATE_180 -> 180
        ExifInterface.ORIENTATION_ROTATE_270 -> 270
        else -> 0
    }
}
