//
//  SDKCallHandler.swift
//  Runner
//
//  Created by Zhenya Knizhnik on 22.08.19.
//  Copyright © 2019 The Chromium Authors. All rights reserved.
//

import Foundation
import Flutter
import ScanbotSDK

class SDKCallHandler {
    func handle(call: FlutterMethodCall, rootViewController: UIViewController?, result: @escaping FlutterResult) {
        switch call.method {
        case "initScanbotSdk": self.handleInitSDK(call: call, result: result)
        case "getLicenseStatus": self.handleLicenseStatus(result: result)
        case "cleanupStorage": self.handleCleanupStorage(result: result)
        case "rotatePageClockwise": self.handleRotatePage(call: call, result: result)
        case "deletePage": self.handleDeletePage(call: call, result: result)
        case "applyImageFilter": self.handleApplyImageFilter(call: call, result: result)
        case "getFilteredDocumentPreviewUri": self.handleGetFilteredDocumentPreviewUri(call: call, result: result)
        case "createPdf": self.handleCreatePDF(call: call, result: result)
        case "createTiff": self.handleCreateTIFF(call: call, result: result)
        case "createPage": self.handleCreatePage(call: call, result: result)
        case "detectDocument": self.handleDetectDocument(call: call, result: result)
        case "performOcr": self.handlePerformOCR(call: call, result: result)
        case "getOcrConfig": self.handleGetOCRConfig(result: result)

        case "startDocumentScanner": self.handleDocumentScanner(call: call, presenter: rootViewController, result: result)
        case "startBarcodeScanner": self.handleBarcodeScanner(call: call, presenter: rootViewController, result: result)
        case "startMrzScanner": self.handleMRZScanner(call: call, presenter: rootViewController, result: result)
        case "startEhicScanner": self.handleEHICScanner(call: call, presenter: rootViewController, result: result)
        case "startCroppingScreen": self.handleCropScreen(call: call, presenter: rootViewController, result: result)
        default: self.handleUnknownCallError(call: call, result: result)
        }
    }
    
    private func handleLicenseStatus(result: @escaping FlutterResult) {
        let status = LicenseStatus(isLicenseValid: ScanbotSDK.isLicenseValid())
        result(status.encodedString)
    }
    
    private func handleInitSDK(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let config = SDKInitializationConfig(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        ScanbotSDKInitializer.initialize(config: config, result: result)
    }
    
    private func handleCleanupStorage(result: @escaping FlutterResult) {
        SBSDKUIPageFileStorage.default().removeAll()
        
        if let url = PluginPreferences.shared.storageCustomFilesURL {
            self.clearDirectory(directoryURL: url)
        }
        
        result(nil)
    }
    
    private func clearDirectory(directoryURL: URL) {
        let fileManager = FileManager.default
        do {
            let urls = try fileManager.contentsOfDirectory(at: directoryURL,
                                                           includingPropertiesForKeys: nil,
                                                           options: FileManager.DirectoryEnumerationOptions())
            for url in urls {
                try fileManager.removeItem(at: url)
            }
        } catch {
            print("Could not clear folder: \(error)")
        }
    }
    
    private func handleGetOCRConfig(result: @escaping FlutterResult) {
        if let config = OCRConfigGetter.encodedOCRConfig {
            result(config)
        } else {
            result(FlutterError(code: "1011", message: "ERROR_iOS: OCRConfig getting error", details: nil))
        }
    }
    
    private func handleDetectDocument(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let page = SBSDKUIPage(from: arguments),
            let image = page.originalImage() else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        
        let polygon = DocumentDetector().detectPolygon(on: image)
        page.polygon = polygon
        
        result(page.encodedString)
    }
    
    private func handlePerformOCR(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let config = OCRPerformConfiguration(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        OCRPerformer().perform(with: config, result: result)
    }
    
    private func handleCreatePDF(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let pageSize = SBSDKPDFRendererPageSize(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        let pages = SBSDKUIPage.pages(from: arguments)
        Renderer().renderPDF(from: pages, pageSize: pageSize, result: result)
    }
    
    private func handleCreateTIFF(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any]  else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        let options = TIFFOptions(from: arguments)
        let pages = SBSDKUIPage.pages(from: arguments)
        Renderer().renderTIFF(from: pages, options: options, result: result)
    }
    
    private func handleCreatePage(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            guard let arguments = call.arguments as? [String: Any],
                let imageURLString = arguments["originalImageFileUri"] as? String,
                let shouldDetect = arguments["shouldDetectDocument"] as? Bool  else {
                    self.handleArgumentsError(call: call, result: result)
                    return
            }
            
            guard let imageURL = URL(string: imageURLString) else {
                self.handleArgumentsError(call: call, result: result)
                return
            }
            
            guard let imageData = try? Data(contentsOf: imageURL),
                let image = UIImage(data: imageData)?.sbsdk_imageWithNormalizedOrientation(),
                let uuid = SBSDKUIPageFileStorage.default().add(image) else {
                    self.handleArgumentsError(call: call, result: result)
                    return
            }
            
            var polygon: SBSDKPolygon?
            if shouldDetect {
                polygon = DocumentDetector().detectPolygon(on: image)
            }
            
            let page = SBSDKUIPage(pageFileID: uuid, polygon: polygon)
            result(page.encodedString)
        }
    }

    
    private func handleRotatePage(call: FlutterMethodCall,result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let page = SBSDKUIPage(from: arguments),
            let count = arguments["rotationsCount"] as? Int else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        page.rotateClockwise(count)
        result(page.encodedString)
    }
    
    private func handleApplyImageFilter(call: FlutterMethodCall,result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let page = SBSDKUIPage(from: arguments),
            let filter = SBSDKImageFilterType(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        page.filter = filter
        result(page.encodedString)
    }
    
    private func handleGetFilteredDocumentPreviewUri(call: FlutterMethodCall,result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let page = SBSDKUIPage(from: arguments),
            let filter = SBSDKImageFilterType(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        let url = page.documentPreviewImageURL(using: filter)
        result(url?.absoluteString)
    }
    
    private func handleDeletePage(call: FlutterMethodCall,result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let page = SBSDKUIPage(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        SBSDKUIPageFileStorage.default().removePageFileID(page.pageFileUUID)
        result(nil)
    }
    
    private lazy var documentScannerPresenter = DocumentScannerPresenter()
    private func handleDocumentScanner(call: FlutterMethodCall,
                                       presenter: UIViewController?,
                                       result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any] else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        let config = SBSDKUIDocumentScannerConfiguration(from: arguments)
        
        if let presenter = presenter {
            self.documentScannerPresenter.present(on: presenter, with: config, result: result)
        }
    }
    
    private lazy var barcodeScannerPresenter = BarcodeScannerPresenter()
    private func handleBarcodeScanner(call: FlutterMethodCall,
                                      presenter: UIViewController?,
                                      result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any] else {
            self.handleArgumentsError(call: call, result: result)
            return
        }
        let config = SBSDKUIMachineCodeScannerConfiguration(from: arguments)
        let codes = AVMetadataObject.ObjectType.fromArguments(arguments)
        
        if let presenter = presenter {
            self.barcodeScannerPresenter.present(on: presenter, with: config, machineCodeTypes: codes, result: result)
        }
    }
    
    private lazy var mrzScannerPresenter = MRZScannerPresenter()
    private func handleMRZScanner(call: FlutterMethodCall,
                                  presenter: UIViewController?,
                                  result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any] else {
            self.handleArgumentsError(call: call, result: result)
            return
        }
        let config = SBSDKUIMachineCodeScannerConfiguration(from: arguments)
        
        if let presenter = presenter {
            self.mrzScannerPresenter.present(on: presenter, with: config, result: result)
        }
    }
    
    private lazy var ehicScannerPresenter = EHICRecognizerPresenter()
    private func handleEHICScanner(call: FlutterMethodCall,
                                  presenter: UIViewController?,
                                  result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any] else {
            self.handleArgumentsError(call: call, result: result)
            return
        }
        let config = SBSDKUIHealthInsuranceCardScannerConfiguration(from: arguments)
        
        if let presenter = presenter {
            self.ehicScannerPresenter.present(on: presenter, with: config, result: result)
        }
    }
    
    private lazy var cropScreenPresenter = CropScreenPresenter()
    private func handleCropScreen(call: FlutterMethodCall,
                                  presenter: UIViewController?,
                                  result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let page = SBSDKUIPage(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        let config = SBSDKUICroppingScreenConfiguration(from: arguments)
        
        if let presenter = presenter {
            self.cropScreenPresenter.present(on: presenter, with: page, configuration: config, result: result)
        }
    }
    
    private func handleArgumentsError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        result(FlutterError(code: "1001",
                            message: "ERROR_iOS: \(call.method) was called with wrong arguments",
                            details: call.arguments))
    }
    
    private func handleUnknownCallError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        result(FlutterError(code: "1002",
                            message: "ERROR_iOS: \(call.method) is unknown or can't be handled",
                            details: nil))
    }
}
