import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:scanbot_sdk/cropping_screen_data.dart';
import 'package:scanbot_sdk/document_scan_data.dart';

import 'barcode_scanning_data.dart';
import 'common_data.dart';
import 'ehic_scanning_data.dart';
import 'mrz_scanning_data.dart';

class ScanbotSdkUi {
  static const MethodChannel _channel = const MethodChannel('scanbot_sdk');

  /// Starts the Ready-to-Use Barcode Scanner UI as full-screen modal UI dialog.
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  static Future<BarcodeScanningResult> startBarcodeScanner(
      BarcodeScannerConfiguration config) async {
    try {
      String barcodeScanningResult =
          await _channel.invokeMethod("startBarcodeScanner", config.toJson());
      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      return BarcodeScanningResult.error(e.toStriJsonUtils.javang());
    }
  }

  /// Starts the Ready-to-Use Document Scanner UI as full-screen modal UI dialog.
  /// The Document Scanner provides guided, automatic document scanning.
  static Future<DocumentScanningResult> startDocumentScanner(
      DocumentScannerConfiguration config) async {
    try {
      var arguments = config.toJson();
      String documentScanningResult =
          await _channel.invokeMethod("startDocumentScanner", arguments);
      return DocumentScanningResult.fromJson(
          jsonDecode(documentScanningResult));
    } catch (e) {
      return DocumentScanningResult.error(e.toString());
    }
  }

  /// Starts the Ready-to-Use MRZ Scanner UI as full-screen modal UI dialog.
  /// The MRZ Scanner is able to recognize and parse data from the Machine Readable Zone
  /// on ID cards, passports, travel documents, etc.
  static Future<MrzScanningResult> startMrzScanner(
      MrzScannerConfiguration config) async {
    try {
      String barcodeScanningResult =
          await _channel.invokeMethod("startMrzScanner", config.toJson());
      return MrzScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      return MrzScanningResult.error(e.toString());
    }
  }

  /// Starts the Ready-to-Use EHIC Scanner UI as full-screen modal UI dialog.
  /// The EHIC Scanner is able to recognize and parse data from the backside of
  /// European Health Insurance Cards.
  static Future<HealthInsuranceCardRecognitionResult> startEhicScanner(
      HealthInsuranceScannerConfiguration config) async {
    try {
      String barcodeScanningResult =
          await _channel.invokeMethod("startEhicScanner", config.toJson());
      return HealthInsuranceCardRecognitionResult.fromJson(
          jsonDecode(barcodeScanningResult));
    } catch (e) {
      return HealthInsuranceCardRecognitionResult(null, null);
    }
  }

  /// Starts the Ready-to-Use Cropping UI as full-screen modal UI dialog.
  /// The Cropping UI provides the functionality for manual cropping and rotation of the given [page].
  static Future<CroppingResult> startCroppingScreen(
      Page page, CroppingScreenConfiguration config) async {
    try {
      var arguments = config.toJson();
      arguments["page"] = page.toJson();
      String documentScanningResult =
          await _channel.invokeMethod("startCroppingScreen", arguments);
      return CroppingResult.fromJson(jsonDecode(documentScanningResult));
    } catch (e) {
      print(e);
      return CroppingResult(null);
    }
  }
}
